/// <reference types="node" />
import { RawSourceMap } from "source-map";
export type Result = {
    data: Buffer;
    map: RawSourceMap;
};
/**
 * @param rebasedPath The rebased path of the asset. When `false`, the asset will not be rebased. When either `null` or `undefined`, the asset will be rebased using the default rebasing logic. When a `string`, the asset will be rebased to that string.
 */
export type RebaseHandlerCallback = (rebasedPath?: false | null | string) => void;
/**
 * @param source The source file where the asset was encountered.
 * @param resolvedPath The resolved path of the asset - i.e. the path of the asset relative to the source file.
 * @param done The callback function to invoke on completion.
 */
export type RebaseHandler = (source: string, resolvedPath: string, done: RebaseHandlerCallback) => void;
export type Options = {
    /**
     * The handler invoked to resolve the rebased path of the asset. Takes precedence over the default rebasing logic.
     */
    rebase?: RebaseHandler;
};
/**
 * @param rebasedPath The rebased path of the rebased asset
 * @param resolvedPath The original path of the rebased asset
 */
export type RebaseEventListener = (rebasedPath: string, resolvedPath: string) => void;
export interface Rebaser {
    /**
     * Rebase the passed HTML markup assets reference.
     *
     * @param html The HTML markup whose assets need to be rebased.
     * @param map The source map that will be used to resolve the assets.
     */
    rebase: (html: Buffer, map: RawSourceMap) => Promise<Result>;
    /**
     * Adds the listener to the end of the listeners array for the event "rebase".
     *
     * @param event
     * @param listener
     */
    on(event: "rebase", listener: RebaseEventListener): this;
}
/**
 * Creates and returns a Rebaser instance.
 *
 * @param options
 */
export declare const createRebaser: (options?: Options) => Rebaser;
