"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createRebaser = void 0;
var parse5_html_rewriting_stream_1 = __importDefault(require("parse5-html-rewriting-stream"));
var source_map_1 = require("source-map");
var events_1 = require("events");
var url_1 = require("url");
var path_1 = require("path");
var slash_1 = __importDefault(require("slash"));
var stream_1 = require("stream");
var css_source_map_rebase_1 = require("css-source-map-rebase");
/**
 * Creates and returns a Rebaser instance.
 *
 * @param options
 */
var createRebaser = function (options) {
    if (options === void 0) { options = {}; }
    var eventEmitter = new events_1.EventEmitter();
    var rebase = function (html, map) {
        var rewritingStream = new parse5_html_rewriting_stream_1.default();
        var rebase = options.rebase;
        var regions = null;
        return new Promise(function (resolve, reject) {
            var data = Buffer.from('');
            var outputStream = new stream_1.Writable({
                write: function (chunk, _encoding, callback) {
                    data = Buffer.concat([data, chunk]);
                    callback();
                }
            });
            outputStream.on('finish', function () {
                resolve({
                    data: data,
                    map: map
                });
            });
            rewritingStream.pipe(outputStream);
            var isRebasable = function (url) {
                return !(0, path_1.isAbsolute)(url.href) && (url.host === null) && ((url.hash === null) || (url.path !== null));
            };
            var queue = Promise.resolve();
            var defer = function (execution) {
                queue = queue
                    .then(execution)
                    .catch(function (error) {
                    reject(error);
                });
            };
            var getRegions = function () {
                if (!regions) {
                    var foundRegions_1 = [];
                    var sourceMapConsumer = new source_map_1.SourceMapConsumer(map);
                    var region_1 = null;
                    var currentSource_1 = null;
                    sourceMapConsumer.eachMapping(function (mapping) {
                        var source = mapping.source;
                        if (source !== currentSource_1) {
                            // end the current region...
                            if (region_1) {
                                region_1.endLine = mapping.generatedLine;
                                region_1.endColumn = mapping.generatedColumn;
                            }
                            //...and start a new one
                            region_1 = {
                                source: source,
                                startLine: mapping.generatedLine,
                                startColumn: mapping.generatedColumn,
                                endLine: null,
                                endColumn: null
                            };
                            foundRegions_1.push(region_1);
                            currentSource_1 = source;
                        }
                    }, null);
                    regions = foundRegions_1;
                }
                return regions;
            };
            var findRegion = function (startLine, startColumn) {
                var i = 0;
                var result = null;
                var regions = getRegions();
                var tagStartLine = startLine;
                var tagStartColumn = startColumn;
                while ((i < regions.length) && (result === null)) {
                    var region = regions[i];
                    if (((region.startLine < tagStartLine) || ((region.startLine === tagStartLine) && (region.startColumn <= tagStartColumn))) &&
                        ((region.endLine === null) || (region.endLine > tagStartLine) ||
                            ((region.endLine === tagStartLine) && (region.endColumn === null || (region.endColumn >= tagStartColumn))))) {
                        result = region;
                    }
                    i++;
                }
                return result;
            };
            var transformText = function (textToken, rawHtml) {
                if ((currentStartTag === null || currentStartTag === void 0 ? void 0 : currentStartTag.tagName) !== "style") {
                    return Promise.resolve();
                }
                var _a = textToken.sourceCodeLocation, startLine = _a.startLine, startCol = _a.startCol, endLine = _a.endLine;
                var numberOfLines = 1 + (endLine - startLine);
                var region = findRegion(startLine, startCol);
                var generator = new source_map_1.SourceMapGenerator();
                for (var generatedLine = 1; generatedLine <= numberOfLines; generatedLine++) {
                    generator.addMapping({
                        source: region.source,
                        generated: {
                            line: generatedLine,
                            column: 0
                        },
                        original: {
                            line: 1,
                            column: 0
                        }
                    });
                }
                generator.setSourceContent(region.source, rawHtml);
                var cssRebaser = new css_source_map_rebase_1.Rebaser({
                    map: Buffer.from(generator.toString()),
                    rebase: rebase
                });
                cssRebaser.on("rebase", function (rebasedPath, resolvedPath) {
                    eventEmitter.emit('rebase', rebasedPath, resolvedPath);
                });
                return cssRebaser.rebase(Buffer.from(rawHtml))
                    .then(function (result) {
                    var css = result.css;
                    textToken.text = css.toString();
                });
            };
            var transformStartTag = function (tag) {
                var processTag = function (tag) {
                    var attributes = tag.attrs;
                    attributes.forEach(function (attribute) {
                        switch (attribute.name) {
                            case 'href':
                            case 'src':
                                var url = (0, url_1.parse)(attribute.value);
                                if (isRebasable(url)) {
                                    var _a = tag.sourceCodeLocation, startLine = _a.startLine, startCol = _a.startCol;
                                    var tagRegion = findRegion(startLine, startCol);
                                    var source = tagRegion.source;
                                    var resolvedPath_1 = path_1.posix.join((0, path_1.dirname)(source), url.pathname);
                                    var done = function (rebasedPath) {
                                        if (rebasedPath !== false) {
                                            if (!rebasedPath) { // default rebasing
                                                rebasedPath = resolvedPath_1;
                                            }
                                            attribute.value = (0, slash_1.default)((0, path_1.join)('.', rebasedPath));
                                            eventEmitter.emit('rebase', rebasedPath, resolvedPath_1);
                                        }
                                    };
                                    if (!rebase) {
                                        rebase = function (_url, _source, done) {
                                            done();
                                        };
                                    }
                                    rebase(source, resolvedPath_1, done);
                                }
                                break;
                        }
                    });
                };
                processTag(tag);
            };
            var currentStartTag = null;
            rewritingStream.on('startTag', function (startTag) {
                defer(function () {
                    currentStartTag = startTag;
                    transformStartTag(startTag);
                    rewritingStream.emitStartTag(startTag);
                    return Promise.resolve();
                });
            });
            rewritingStream.on('text', function (text, rawHtml) {
                defer(function () {
                    return transformText(text, rawHtml)
                        .then(function () {
                        if (rewritingStream.writable) {
                            rewritingStream.emitRaw(text.text);
                        }
                        else {
                            outputStream.write(text.text);
                        }
                    });
                });
            });
            rewritingStream.on("endTag", function (endTag) {
                defer(function () {
                    currentStartTag = null;
                    rewritingStream.emitEndTag(endTag);
                    return Promise.resolve();
                });
            });
            for (var _i = 0, _a = ['doctype', 'comment']; _i < _a.length; _i++) {
                var eventName = _a[_i];
                rewritingStream.on(eventName, function (_token, rawHtml) {
                    defer(function () {
                        rewritingStream.emitRaw(rawHtml);
                        return Promise.resolve();
                    });
                });
            }
            rewritingStream.write(html.toString(), function () {
                queue.then(function () {
                    rewritingStream.end();
                });
            });
        });
    };
    var rebaser = {
        rebase: rebase,
        on: function (event, listener) {
            eventEmitter.on(event, listener);
            return rebaser;
        }
    };
    return rebaser;
};
exports.createRebaser = createRebaser;
